@@ -22,6 +22,10 @@ struct WebPageStickerSet;
 enum class WebPageType : uint8;
 enum class NewMessageType;
 
+namespace Calls {
+class GroupCall;
+} // namespace Calls
+
 namespace HistoryView {
 struct Group;
 class Element;
@@ -121,6 +125,16 @@ struct SentFromScheduled {
 	MsgId sentId = 0;
 };
 
+struct RecentSelfForwards {
+	PeerId fromPeerId = 0;
+	MessageIdsList ids;
+};
+
+struct RecentJoinChat {
+	PeerId fromPeerId = 0;
+	PeerId joinedPeerId = 0;
+};
+
 class Session final {
 public:
 	using ViewElement = HistoryView::Element;
@@ -375,6 +389,8 @@ public:
 	[[nodiscard]] rpl::producer<not_null<History*>> historyChanged() const;
 	void notifyViewPaidReactionSent(not_null<const ViewElement*> view);
 	[[nodiscard]] rpl::producer<not_null<const ViewElement*>> viewPaidReactionSent() const;
+	void notifyCallPaidReactionSent(not_null<Calls::GroupCall*> call);
+	[[nodiscard]] rpl::producer<not_null<Calls::GroupCall*>> callPaidReactionSent() const;
 	void sendHistoryChangeNotifications();
 
 	void notifyPinnedDialogsOrderUpdated();
@@ -579,7 +595,9 @@ public:
 		MessageFlags localFlags,
 		NewMessageType type);
 
+	[[nodiscard]] int unreadBadge() const;
 	[[nodiscard]] int unreadWithMentionsBadge() const;
+	[[nodiscard]] bool unreadBadgeMuted() const;
 	[[nodiscard]] bool unreadWithMentionsBadgeMuted() const;
 	[[nodiscard]] int unreadBadgeIgnoreOne(Dialogs::Key key) const;
 	[[nodiscard]] bool unreadBadgeMutedIgnoreOne(Dialogs::Key key) const;
@@ -885,6 +903,12 @@ public:
 	void setPendingStarsRating(StarsRatingPending value);
 	[[nodiscard]] StarsRatingPending pendingStarsRating() const;
 
+	void addRecentSelfForwards(const RecentSelfForwards &data);
+	[[nodiscard]] rpl::producer<RecentSelfForwards> recentSelfForwards() const;
+
+	void addRecentJoinChat(const RecentJoinChat &data);
+	[[nodiscard]] rpl::producer<RecentJoinChat> recentJoinChat() const;
+
 	void clearLocalStorage();
 
 private:
@@ -1061,6 +1085,7 @@ private:
 	rpl::event_stream<not_null<const HistoryItem*>> _itemRemoved;
 	rpl::event_stream<not_null<const ViewElement*>> _viewRemoved;
 	rpl::event_stream<not_null<const ViewElement*>> _viewPaidReactionSent;
+	rpl::event_stream<not_null<Calls::GroupCall*>> _callPaidReactionSent;
 	rpl::event_stream<not_null<const History*>> _historyUnloaded;
 	rpl::event_stream<not_null<const History*>> _historyCleared;
 	base::flat_set<not_null<History*>> _historiesChanged;
@@ -1253,6 +1278,9 @@ private:
 		not_null<PeerData*>,
 		NextToUpgradeGift> _nextForUpgradeGifts;
 
+	rpl::event_stream<RecentSelfForwards> _recentSelfForwards;
+	rpl::event_stream<RecentJoinChat> _recentJoinChat;
+
 	rpl::lifetime _lifetime;
 
 };
