@@ -9,25 +9,71 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "ui/chat/message_bubble.h"
 #include "ui/chat/chat_style.h"
+#include "ui/effects/reaction_fly_animation.h"
 #include "ui/text/text_options.h"
 #include "ui/text/text_utilities.h"
+#include "ui/painter.h"
+#include "core/ui_integration.h"
 #include "lang/lang_keys.h"
 #include "history/history_item_components.h"
-#include "history/history_message.h"
+#include "history/history_item.h"
 #include "history/history.h"
+#include "history/view/media/history_view_media.h"
 #include "history/view/history_view_message.h"
 #include "history/view/history_view_cursor_state.h"
-#include "history/view/history_view_react_animation.h"
+#include "base/unixtime.h"
+#include "chat_helpers/emoji_interactions.h"
 #include "core/click_handler_types.h"
 #include "main/main_session.h"
 #include "lottie/lottie_icon.h"
+#include "data/data_channel.h"
 #include "data/data_session.h"
 #include "data/data_message_reactions.h"
 #include "window/window_session_controller.h"
 #include "styles/style_chat.h"
+#include "styles/style_credits.h"
 #include "styles/style_dialogs.h"
 
 namespace HistoryView {
+namespace {
+
+[[nodiscard]] QString SchedulePeriodText(TimeId period) {
+	struct Entry {
+		TimeId period = 0;
+		QString text;
+	};
+	const auto map = std::vector<Entry>{
+		{ 60, u"minutely"_q },
+		{ 300, u"5-minutely"_q },
+		{ 24 * 60 * 60, tr::lng_repeated_daily(tr::now) },
+		{ 7 * 24 * 60 * 60, tr::lng_repeated_weekly(tr::now) },
+		{ 14 * 24 * 60 * 60, tr::lng_repeated_biweekly(tr::now) },
+		{ 30 * 24 * 60 * 60, tr::lng_repeated_monthly(tr::now) },
+		{
+			91 * 24 * 60 * 60,
+			tr::lng_repeated_every_month(tr::now, lt_count, 3)
+		},
+		{
+			182 * 24 * 60 * 60,
+			tr::lng_repeated_every_month(tr::now, lt_count, 6)
+		},
+		{ 365 * 24 * 60 * 60, tr::lng_repeated_yearly(tr::now) },
+	};
+	for (const auto &entry : map) {
+		if (entry.period >= period) {
+			return entry.text;
+		}
+	}
+	return map.back().text;
+}
+
+} // namespace
+
+struct BottomInfo::Effect {
+	mutable std::unique_ptr<Ui::ReactionFlyAnimation> animation;
+	mutable QImage image;
+	EffectId id = 0;
+};
 
 BottomInfo::BottomInfo(
 	not_null<::Data::Reactions*> reactionsOwner,
@@ -47,17 +93,11 @@ void BottomInfo::update(Data &&data, int availableWidth) {
 	}
 }
 
-int BottomInfo::countReactionsMaxWidth() const {
+int BottomInfo::countEffectMaxWidth() const {
 	auto result = 0;
-	for (const auto &reaction : _reactions) {
+	if (_effect) {
 		result += st::reactionInfoSize;
-		if (reaction.countTextWidth > 0) {
-			result += st::reactionInfoSkip
-				+ reaction.countTextWidth
-				+ st::reactionInfoDigitSkip;
-		} else {
-			result += st::reactionInfoBetween;
-		}
+		result += st::reactionInfoBetween;
 	}
 	if (result) {
 		result += (st::reactionInfoSkip - st::reactionInfoBetween);
@@ -65,19 +105,14 @@ int BottomInfo::countReactionsMaxWidth() const {
 	return result;
 }
 
-int BottomInfo::countReactionsHeight(int newWidth) const {
+int BottomInfo::countEffectHeight(int newWidth) const {
 	const auto left = 0;
 	auto x = 0;
 	auto y = 0;
 	auto widthLeft = newWidth;
-	for (const auto &reaction : _reactions) {
-		const auto add = (reaction.countTextWidth > 0)
-			? st::reactionInfoDigitSkip
-			: st::reactionInfoBetween;
-		const auto width = st::reactionInfoSize
-			+ (reaction.countTextWidth > 0
-				? (st::reactionInfoSkip + reaction.countTextWidth)
-				: 0);
+	if (_effect) {
+		const auto add = st::reactionInfoBetween;
+		const auto width = st::reactionInfoSize;
 		if (x > left && widthLeft < width) {
 			x = left;
 			y += st::msgDateFont->height;
@@ -96,29 +131,65 @@ int BottomInfo::firstLineWidth() const {
 	if (height() == minHeight()) {
 		return width();
 	}
-	return maxWidth() - _reactionsMaxWidth;
+	return maxWidth() - _effectMaxWidth;
 }
 
 bool BottomInfo::isWide() const {
 	return (_data.flags & Data::Flag::Edited)
+		|| _data.scheduleRepeatPeriod
 		|| !_data.author.isEmpty()
 		|| !_views.isEmpty()
 		|| !_replies.isEmpty()
-		|| !_reactions.empty();
+		|| _effect;
 }
 
 TextState BottomInfo::textState(
-		not_null<const HistoryItem*> item,
+		not_null<const Message*> view,
 		QPoint position) const {
+	const auto item = view->data();
 	auto result = TextState(item);
-	if (const auto link = revokeReactionLink(item, position)) {
+	if (const auto link = replayEffectLink(view, position)) {
 		result.link = link;
 		return result;
 	}
+	const auto textWidth = _authorEditedDate.maxWidth();
+	auto withTicksWidth = textWidth;
+	if (_data.flags & (Data::Flag::OutLayout | Data::Flag::Sending)) {
+		withTicksWidth += st::historySendStateSpace;
+	}
+	if (!_views.isEmpty()) {
+		const auto viewsWidth = _views.maxWidth();
+		const auto right = width()
+			- withTicksWidth
+			- ((_data.flags & Data::Flag::Pinned) ? st::historyPinWidth : 0)
+			- st::historyViewsSpace
+			- st::historyViewsWidth
+			- viewsWidth;
+		const auto inViews = QRect(
+			right,
+			0,
+			withTicksWidth + st::historyViewsWidth,
+			st::msgDateFont->height
+		).contains(position);
+		if (inViews) {
+			result.customTooltip = true;
+			const auto fullViews = tr::lng_views_tooltip(
+				tr::now,
+				lt_count_decimal,
+				*_data.views);
+			const auto fullForwards = _data.forwardsCount
+				? ('\n' + tr::lng_forwards_tooltip(
+					tr::now,
+					lt_count_decimal,
+					*_data.forwardsCount))
+				: QString();
+			result.customTooltipText = fullViews + fullForwards;
+		}
+	}
 	const auto inTime = QRect(
-		width() - _dateWidth,
+		width() - withTicksWidth,
 		0,
-		_dateWidth,
+		withTicksWidth,
 		st::msgDateFont->height
 	).contains(position);
 	if (inTime) {
@@ -127,70 +198,44 @@ TextState BottomInfo::textState(
 	return result;
 }
 
-ClickHandlerPtr BottomInfo::revokeReactionLink(
-		not_null<const HistoryItem*> item,
+ClickHandlerPtr BottomInfo::replayEffectLink(
+		not_null<const Message*> view,
 		QPoint position) const {
-	if (_reactions.empty()) {
+	if (!_effect) {
 		return nullptr;
 	}
 	auto left = 0;
 	auto top = 0;
 	auto available = width();
 	if (height() != minHeight()) {
-		available = std::min(available, _reactionsMaxWidth);
+		available = std::min(available, _effectMaxWidth);
 		left += width() - available;
 		top += st::msgDateFont->height;
 	}
-	auto x = left;
-	auto y = top;
-	auto widthLeft = available;
-	for (const auto &reaction : _reactions) {
-		const auto chosen = (reaction.emoji == _data.chosenReaction);
-		const auto add = (reaction.countTextWidth > 0)
-			? st::reactionInfoDigitSkip
-			: st::reactionInfoBetween;
-		const auto width = st::reactionInfoSize
-			+ (reaction.countTextWidth > 0
-				? (st::reactionInfoSkip + reaction.countTextWidth)
-				: 0);
-		if (x > left && widthLeft < width) {
-			x = left;
-			y += st::msgDateFont->height;
-			widthLeft = available;
-		}
+	if (_effect) {
 		const auto image = QRect(
-			x,
-			y,
+			left,
+			top,
 			st::reactionInfoSize,
 			st::msgDateFont->height);
-		if (chosen && image.contains(position)) {
-			if (!_revokeLink) {
-				_revokeLink = revokeReactionLink(item);
+		if (image.contains(position)) {
+			if (!_replayLink) {
+				_replayLink = replayEffectLink(view);
 			}
-			return _revokeLink;
+			return _replayLink;
 		}
-		x += width + add;
-		widthLeft -= width + add;
 	}
 	return nullptr;
 }
 
-ClickHandlerPtr BottomInfo::revokeReactionLink(
-		not_null<const HistoryItem*> item) const {
-	const auto itemId = item->fullId();
-	const auto sessionId = item->history()->session().uniqueId();
-	return std::make_shared<LambdaClickHandler>([=](
-			ClickContext context) {
+ClickHandlerPtr BottomInfo::replayEffectLink(
+		not_null<const Message*> view) const {
+	const auto weak = base::make_weak(view);
+	return std::make_shared<LambdaClickHandler>([=](ClickContext context) {
 		const auto my = context.other.value<ClickHandlerContext>();
-		if (const auto controller = my.sessionWindow.get()) {
-			if (controller->session().uniqueId() == sessionId) {
-				auto &owner = controller->session().data();
-				if (const auto item = owner.message(itemId)) {
-					const auto chosen = item->chosenReaction();
-					if (!chosen.isEmpty()) {
-						item->toggleReaction(chosen);
-					}
-				}
+		if ([[maybe_unused]] const auto controller = my.sessionWindow.get()) {
+			if (const auto strong = weak.get()) {
+				strong->delegate()->elementStartEffect(strong, nullptr);
 			}
 		}
 	});
@@ -293,20 +338,20 @@ void BottomInfo::paint(
 			firstLineBottom + st::historyViewsTop,
 			outerWidth);
 	}
-	if (!_reactions.empty()) {
+	if (_effect) {
 		auto left = position.x();
 		auto top = position.y();
 		auto available = width();
 		if (height() != minHeight()) {
-			available = std::min(available, _reactionsMaxWidth);
+			available = std::min(available, _effectMaxWidth);
 			left += width() - available;
 			top += st::msgDateFont->height;
 		}
-		paintReactions(p, position, left, top, available, context);
+		paintEffect(p, position, left, top, available, context);
 	}
 }
 
-void BottomInfo::paintReactions(
+void BottomInfo::paintEffect(
 		Painter &p,
 		QPoint origin,
 		int left,
@@ -314,7 +359,7 @@ void BottomInfo::paintReactions(
 		int availableWidth,
 		const PaintContext &context) const {
 	struct SingleAnimation {
-		not_null<Reactions::Animation*> animation;
+		not_null<Ui::ReactionFlyAnimation*> animation;
 		QRect target;
 	};
 	std::vector<SingleAnimation> animations;
@@ -322,63 +367,52 @@ void BottomInfo::paintReactions(
 	auto x = left;
 	auto y = top;
 	auto widthLeft = availableWidth;
-	for (const auto &reaction : _reactions) {
-		if (context.reactionInfo
-			&& reaction.animation
-			&& reaction.animation->finished()) {
-			reaction.animation = nullptr;
-		}
-		const auto animating = (reaction.animation != nullptr);
-		const auto add = (reaction.countTextWidth > 0)
-			? st::reactionInfoDigitSkip
-			: st::reactionInfoBetween;
-		const auto width = st::reactionInfoSize
-			+ (reaction.countTextWidth > 0
-				? (st::reactionInfoSkip + reaction.countTextWidth)
-				: 0);
+	if (_effect) {
+		const auto animating = (_effect->animation != nullptr);
+		const auto add = st::reactionInfoBetween;
+		const auto width = st::reactionInfoSize;
 		if (x > left && widthLeft < width) {
 			x = left;
 			y += st::msgDateFont->height;
 			widthLeft = availableWidth;
 		}
-		if (reaction.image.isNull()) {
-			reaction.image = _reactionsOwner->resolveImageFor(
-				reaction.emoji,
-				::Data::Reactions::ImageSize::BottomInfo);
+		if (_effect->image.isNull()) {
+			_effect->image = _reactionsOwner->resolveEffectImageFor(
+				_effect->id);
 		}
 		const auto image = QRect(
-			x + (st::reactionInfoSize - st::reactionInfoImage) / 2,
-			y + (st::msgDateFont->height - st::reactionInfoImage) / 2,
-			st::reactionInfoImage,
-			st::reactionInfoImage);
-		const auto skipImage = animating
-			&& (reaction.count < 2 || !reaction.animation->flying());
-		if (!reaction.image.isNull() && !skipImage) {
-			p.drawImage(image.topLeft(), reaction.image);
+			x + (st::reactionInfoSize - st::effectInfoImage) / 2,
+			y + (st::msgDateFont->height - st::effectInfoImage) / 2,
+			st::effectInfoImage,
+			st::effectInfoImage);
+		if (!_effect->image.isNull()) {
+			p.drawImage(image.topLeft(), _effect->image);
 		}
 		if (animating) {
 			animations.push_back({
-				.animation = reaction.animation.get(),
+				.animation = _effect->animation.get(),
 				.target = image,
 			});
 		}
-		if (reaction.countTextWidth > 0) {
-			p.drawText(
-				x + st::reactionInfoSize + st::reactionInfoSkip,
-				y + st::msgDateFont->ascent,
-				reaction.countText);
-		}
 		x += width + add;
 		widthLeft -= width + add;
 	}
 	if (!animations.empty()) {
-		context.reactionInfo->effectPaint = [=](QPainter &p) {
+		const auto now = context.now;
+		context.reactionInfo->effectPaint = [
+			now,
+			origin,
+			list = std::move(animations)
+		](QPainter &p) {
 			auto result = QRect();
-			for (const auto &single : animations) {
+			for (const auto &single : list) {
 				const auto area = single.animation->paintGetArea(
 					p,
 					origin,
-					single.target);
+					single.target,
+					QColor(255, 255, 255, 0), // Colored, for emoji status.
+					QRect(), // Clip, for emoji status.
+					now);
 				result = result.isEmpty() ? area : result.united(area);
 			}
 			return result;
@@ -387,32 +421,40 @@ void BottomInfo::paintReactions(
 }
 
 QSize BottomInfo::countCurrentSize(int newWidth) {
-	if (newWidth >= maxWidth()) {
+	if (newWidth >= maxWidth() || (_data.flags & Data::Flag::Shortcut)) {
 		return optimalSize();
 	}
-	const auto noReactionsWidth = maxWidth() - _reactionsMaxWidth;
-	accumulate_min(newWidth, std::max(noReactionsWidth, _reactionsMaxWidth));
+	const auto dateHeight = (_data.flags & Data::Flag::Sponsored)
+		? 0
+		: st::msgDateFont->height;
+	const auto noReactionsWidth = maxWidth() - _effectMaxWidth;
+	accumulate_min(newWidth, std::max(noReactionsWidth, _effectMaxWidth));
 	return QSize(
 		newWidth,
-		st::msgDateFont->height + countReactionsHeight(newWidth));
+		dateHeight + countEffectHeight(newWidth));
 }
 
 void BottomInfo::layout() {
 	layoutDateText();
 	layoutViewsText();
 	layoutRepliesText();
-	layoutReactionsText();
+	layoutEffectText();
 	initDimensions();
 }
 
 void BottomInfo::layoutDateText() {
 	const auto edited = (_data.flags & Data::Flag::Edited)
 		? (tr::lng_edited(tr::now) + ' ')
+		: (_data.flags & Data::Flag::EstimateDate)
+		? (tr::lng_approximate(tr::now) + ' ')
+		: _data.scheduleRepeatPeriod
+		? (SchedulePeriodText(_data.scheduleRepeatPeriod) + ' ')
 		: QString();
 	const auto author = _data.author;
-	const auto prefix = !author.isEmpty() ? qsl(", ") : QString();
-	const auto date = edited + _data.date.toString(cTimeFormat());
-	_dateWidth = st::msgDateFont->width(date);
+	const auto prefix = !author.isEmpty() ? u", "_q : QString();
+	const auto date = edited + ((_data.flags & Data::Flag::ForwardedDate)
+		? langDateTime(_data.date)
+		: QLocale().toString(_data.date.time(), QLocale::ShortFormat));
 	const auto afterAuthor = prefix + date;
 	const auto afterAuthorWidth = st::msgDateFont->width(afterAuthor);
 	const auto authorWidth = st::msgDateFont->width(author);
@@ -423,12 +465,24 @@ void BottomInfo::layoutDateText() {
 		? st::msgDateFont->elided(author, maxWidth - afterAuthorWidth)
 		: author;
 	const auto full = (_data.flags & Data::Flag::Sponsored)
-		? tr::lng_sponsored(tr::now)
-		: name.isEmpty() ? date : (name + afterAuthor);
-	_authorEditedDate.setText(
+		? QString()
+		: (_data.flags & Data::Flag::Imported)
+		? (date + ' ' + tr::lng_imported(tr::now))
+		: name.isEmpty()
+		? date
+		: (name + afterAuthor);
+	auto marked = TextWithEntities();
+	if (const auto count = _data.stars) {
+		marked.append(
+			Ui::Text::IconEmoji(&st::starIconEmojiSmall)
+		).append(Lang::FormatCountToShort(count).string).append(u", "_q);
+	}
+	marked.append(full);
+	_authorEditedDate.setMarkedText(
 		st::msgDateTextStyle,
-		full,
-		Ui::NameTextOptions());
+		marked,
+		Ui::NameTextOptions(),
+		Core::TextContext({ .session = &_reactionsOwner->session() }));
 }
 
 void BottomInfo::layoutViewsText() {
@@ -446,7 +500,8 @@ void BottomInfo::layoutRepliesText() {
 	if (!_data.replies
 		|| !*_data.replies
 		|| (_data.flags & Data::Flag::RepliesContext)
-		|| (_data.flags & Data::Flag::Sending)) {
+		|| (_data.flags & Data::Flag::Sending)
+		|| (_data.flags & Data::Flag::Shortcut)) {
 		_replies.clear();
 		return;
 	}
@@ -456,31 +511,18 @@ void BottomInfo::layoutRepliesText() {
 		Ui::NameTextOptions());
 }
 
-void BottomInfo::layoutReactionsText() {
-	if (_data.reactions.empty()) {
-		_reactions.clear();
+void BottomInfo::layoutEffectText() {
+	if (!_data.effectId) {
+		_effect = nullptr;
 		return;
 	}
-	auto sorted = ranges::view::all(
-		_data.reactions
-	) | ranges::view::transform([](const auto &pair) {
-		return std::make_pair(pair.first, pair.second);
-	}) | ranges::to_vector;
-	ranges::sort(sorted, std::greater<>(), &std::pair<QString, int>::second);
-
-	auto reactions = std::vector<Reaction>();
-	reactions.reserve(sorted.size());
-	for (const auto &[emoji, count] : sorted) {
-		const auto i = ranges::find(_reactions, emoji, &Reaction::emoji);
-		reactions.push_back((i != end(_reactions))
-			? std::move(*i)
-			: prepareReactionWithEmoji(emoji));
-		setReactionCount(reactions.back(), count);
-	}
-	_reactions = std::move(reactions);
+	_effect = std::make_unique<Effect>(prepareEffectWithId(_data.effectId));
 }
 
 QSize BottomInfo::countOptimalSize() {
+	if (_data.flags & Data::Flag::Shortcut) {
+		return { st::historyShortcutStateSpace, st::msgDateFont->height };
+	}
 	auto width = 0;
 	if (_data.flags & (Data::Flag::OutLayout | Data::Flag::Sending)) {
 		width += st::historySendStateSpace;
@@ -499,79 +541,58 @@ QSize BottomInfo::countOptimalSize() {
 	if (_data.flags & Data::Flag::Pinned) {
 		width += st::historyPinWidth;
 	}
-	_reactionsMaxWidth = countReactionsMaxWidth();
-	width += _reactionsMaxWidth;
-	return QSize(width, st::msgDateFont->height);
+	_effectMaxWidth = countEffectMaxWidth();
+	width += _effectMaxWidth;
+	const auto dateHeight = (_data.flags & Data::Flag::Sponsored)
+		? 0
+		: st::msgDateFont->height;
+	return QSize(width, dateHeight);
 }
 
-BottomInfo::Reaction BottomInfo::prepareReactionWithEmoji(
-		const QString &emoji) {
-	auto result = Reaction{ .emoji = emoji };
-	_reactionsOwner->preloadImageFor(emoji);
+BottomInfo::Effect BottomInfo::prepareEffectWithId(EffectId id) {
+	auto result = Effect{ .id = id };
+	_reactionsOwner->preloadEffectImageFor(id);
 	return result;
 }
 
-void BottomInfo::setReactionCount(Reaction &reaction, int count) {
-	if (reaction.count == count) {
-		return;
-	}
-	reaction.count = count;
-	reaction.countText = (count > 1)
-		? Lang::FormatCountToShort(count).string
-		: QString();
-	reaction.countTextWidth = (count > 1)
-		? st::msgDateFont->width(reaction.countText)
-		: 0;
+auto BottomInfo::takeEffectAnimation()
+-> std::unique_ptr<Ui::ReactionFlyAnimation> {
+	return _effect ? std::move(_effect->animation) : nullptr;
 }
 
-void BottomInfo::animateReaction(
-		ReactionAnimationArgs &&args,
-		Fn<void()> repaint) {
-	const auto i = ranges::find(_reactions, args.emoji, &Reaction::emoji);
-	if (i == end(_reactions)) {
-		return;
+void BottomInfo::continueEffectAnimation(
+		std::unique_ptr<Ui::ReactionFlyAnimation> animation) {
+	if (_effect) {
+		_effect->animation = std::move(animation);
 	}
-	i->animation = std::make_unique<Reactions::Animation>(
-		_reactionsOwner,
-		args.translated(QPoint(width(), height())),
-		std::move(repaint),
-		st::reactionInfoImage);
 }
 
-auto BottomInfo::takeReactionAnimations()
--> base::flat_map<QString, std::unique_ptr<Reactions::Animation>> {
-	auto result = base::flat_map<
-		QString,
-		std::unique_ptr<Reactions::Animation>>();
-	for (auto &reaction : _reactions) {
-		if (reaction.animation) {
-			result.emplace(reaction.emoji, std::move(reaction.animation));
-		}
+QRect BottomInfo::effectIconGeometry() const {
+	if (!_effect) {
+		return {};
 	}
-	return result;
-}
-
-void BottomInfo::continueReactionAnimations(base::flat_map<
-		QString,
-		std::unique_ptr<Reactions::Animation>> animations) {
-	for (auto &[emoji, animation] : animations) {
-		const auto i = ranges::find(_reactions, emoji, &Reaction::emoji);
-		if (i != end(_reactions)) {
-			i->animation = std::move(animation);
-		}
+	auto left = 0;
+	auto top = 0;
+	auto available = width();
+	if (height() != minHeight()) {
+		available = std::min(available, _effectMaxWidth);
+		left += width() - available;
+		top += st::msgDateFont->height;
 	}
+	return QRect(
+		left + (st::reactionInfoSize - st::effectInfoImage) / 2,
+		top + (st::msgDateFont->height - st::effectInfoImage) / 2,
+		st::effectInfoImage,
+		st::effectInfoImage);
 }
 
 BottomInfo::Data BottomInfoDataFromMessage(not_null<Message*> message) {
 	using Flag = BottomInfo::Data::Flag;
-	const auto item = message->message();
+	const auto item = message->data();
 
 	auto result = BottomInfo::Data();
 	result.date = message->dateTime();
-	if (message->embedReactionsInBottomInfo()) {
-		result.reactions = item->reactions();
-		result.chosenReaction = item->chosenReaction();
-	}
+	result.effectId = item->effectId();
 	if (message->hasOutLayout()) {
 		result.flags |= Flag::OutLayout;
 	}
@@ -584,16 +605,21 @@ BottomInfo::Data BottomInfoDataFromMessage(not_null<Message*> message) {
 	if (item->isPinned() && message->context() != Context::Pinned) {
 		result.flags |= Flag::Pinned;
 	}
-	if (const auto msgsigned = item->Get<HistoryMessageSigned>()) {
-		 if (!msgsigned->isAnonymousRank) {
-			result.author = msgsigned->author;
-		 }
+	if (message->context() == Context::ShortcutMessages) {
+		result.flags |= Flag::Shortcut;
 	}
-	if (!item->hideEditedBadge()) {
-		if (const auto edited = message->displayedEditBadge()) {
-			result.flags |= Flag::Edited;
+	if (!item->isPost()
+		|| !item->hasRealFromId()
+		|| !item->history()->peer->asChannel()->signatureProfiles()) {
+		if (const auto msgsigned = item->Get<HistoryMessageSigned>()) {
+			if (!msgsigned->isAnonymousRank) {
+				result.author = msgsigned->author;
+			}
 		}
 	}
+	if (message->displayedEditDate()) {
+		result.flags |= Flag::Edited;
+	}
 	if (const auto views = item->Get<HistoryMessageViews>()) {
 		if (views->views.count >= 0) {
 			result.views = views->views.count;
@@ -601,11 +627,42 @@ BottomInfo::Data BottomInfoDataFromMessage(not_null<Message*> message) {
 		if (views->replies.count >= 0 && !views->commentsMegagroupId) {
 			result.replies = views->replies.count;
 		}
+		if (views->forwardsCount > 0) {
+			result.forwardsCount = views->forwardsCount;
+		}
 	}
 	if (item->isSending() || item->hasFailed()) {
 		result.flags |= Flag::Sending;
 	}
-	// We don't want to pass and update it in Date for now.
+	if (!item->history()->peer->isUser()) {
+		const auto media = message->media();
+		const auto mine = PaidInformation{
+			.messages = 1,
+			.stars = item->starsPaid(),
+		};
+		auto info = media ? media->paidInformation().value_or(mine) : mine;
+		if (const auto total = info.stars) {
+			result.stars = total;
+		}
+	}
+	const auto forwarded = item->Get<HistoryMessageForwarded>();
+	if (forwarded && forwarded->imported) {
+		result.flags |= Flag::Imported;
+	}
+	if (item->awaitingVideoProcessing()) {
+		result.flags |= Flag::EstimateDate;
+	}
+	if (item->isScheduled()) {
+		result.scheduleRepeatPeriod = item->scheduleRepeatPeriod();
+	}
+	if (forwarded
+			&& forwarded->savedFromPeer
+			&& forwarded->savedFromMsgId
+			&& !item->externalReply()) {
+		result.date = base::unixtime::parse(forwarded->originalDate);
+		result.flags |= Flag::ForwardedDate;
+	}
+	// We don't want to pass and update it in Data for now.
 	//if (item->unread()) {
 	//	result.flags |= Flag::Unread;
 	//}
