@@ -12,6 +12,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "base/unixtime.h"
 #include "boxes/abstract_box.h"
 #include "boxes/premium_preview_box.h"
+#include "calls/group/calls_group_stars_box.h"
 #include "chat_helpers/compose/compose_show.h"
 #include "chat_helpers/stickers_emoji_pack.h"
 #include "core/shortcuts.h"
@@ -670,9 +671,45 @@ FillMenuResult AttachSendMenuEffect(
 	return FillMenuResult::Prepared;
 }
 
+FillMenuResult FillEditCommentPriceMenu(
+		not_null<Ui::PopupMenu*> menu,
+		std::shared_ptr<ChatHelpers::Show> show,
+		Details details,
+		Fn<void(Action, Details)> action,
+		const style::ComposeIcons *iconsOverride,
+		std::optional<QPoint> desiredPositionOverride) {
+	const auto &icons = iconsOverride
+		? *iconsOverride
+		: st::defaultComposeIcons;
+	menu->addAction(tr::lng_video_stream_edit_stars(tr::now), [=] {
+		show->show(Calls::Group::MakeVideoStreamStarsBox({
+			.show = show,
+			.min = int(details.commentPriceMin.value_or(1)),
+			.current = int(details.price.value_or(1)),
+			.save = [=](int count) {
+				auto copy = details;
+				copy.price = count;
+				action({ {}, Action::Type::ChangePrice }, copy);
+			},
+			.name = details.commentStreamerName,
+			//.preview = details.commentPreview,
+		}));
+	}, &icons.menuEditStars);
+	if (details.price.value_or(0) > details.commentPriceMin.value_or(0)) {
+		auto copy = details;
+		copy.price = details.commentPriceMin.value_or(0);
+		menu->addAction(tr::lng_video_stream_remove_stars(tr::now), [=] {
+			action({ {}, Action::Type::ChangePrice }, copy);
+		}, &icons.menuGifRemove);
+	}
+	const auto position = desiredPositionOverride.value_or(QCursor::pos());
+	menu->prepareGeometryFor(position);
+	return FillMenuResult::Prepared;
+}
+
 FillMenuResult FillSendMenu(
 		not_null<Ui::PopupMenu*> menu,
-		std::shared_ptr<ChatHelpers::Show> showForEffect,
+		std::shared_ptr<ChatHelpers::Show> show,
 		Details details,
 		Fn<void(Action, Details)> action,
 		const style::ComposeIcons *iconsOverride,
@@ -685,6 +722,14 @@ FillMenuResult FillSendMenu(
 		&& !details.price.has_value();
 	if (empty || !action) {
 		return FillMenuResult::Skipped;
+	} else if (type == Type::EditCommentPrice) {
+		return FillEditCommentPriceMenu(
+			menu,
+			show,
+			details,
+			action,
+			iconsOverride,
+			desiredPositionOverride);
 	}
 	const auto &icons = iconsOverride
 		? *iconsOverride
@@ -698,7 +743,7 @@ FillMenuResult FillSendMenu(
 	}
 	if (sending && type != Type::SilentOnly) {
 		menu->addAction(
-			(type == Type::Reminder
+			((type == Type::Reminder)
 				? tr::lng_reminder_message(tr::now)
 				: tr::lng_schedule_message(tr::now)),
 			[=] { action({ .type = ActionType::Schedule }, details); },
@@ -752,10 +797,10 @@ FillMenuResult FillSendMenu(
 			&icons.menuPrice);
 	}
 
-	if (showForEffect) {
+	if (show) {
 		return AttachSendMenuEffect(
 			menu,
-			showForEffect,
+			show,
 			details,
 			action,
 			desiredPositionOverride);
@@ -769,13 +814,20 @@ void SetupMenuAndShortcuts(
 		not_null<Ui::RpWidget*> button,
 		std::shared_ptr<ChatHelpers::Show> show,
 		Fn<Details()> details,
-		Fn<void(Action, Details)> action) {
+		Fn<void(Action, Details)> action,
+		const style::PopupMenu *stOverride,
+		const style::ComposeIcons *iconsOverride) {
 	const auto menu = std::make_shared<base::unique_qptr<Ui::PopupMenu>>();
 	const auto showMenu = [=] {
 		*menu = base::make_unique_q<Ui::PopupMenu>(
 			button,
-			st::popupMenuWithIcons);
-		const auto result = FillSendMenu(*menu, show, details(), action);
+			stOverride ? *stOverride : st::popupMenuWithIcons);
+		const auto result = FillSendMenu(
+			*menu,
+			show,
+			details(),
+			action,
+			iconsOverride);
 		if (result != FillMenuResult::Prepared) {
 			return false;
 		}
